@@ -1,6 +1,7 @@
 // allowed.h: externs for allowed.c
 
-void InitAllowedFunc(SEXP Allowed, SEXP Env, int nPreds);
+void InitAllowedFunc(SEXP Allowed, int nAllowedFuncArgs, SEXP Env, 
+                     const char *sPredNames[], int nPreds);
 
 void FreeAllowedFunc(void);
 
